/*
 * Virtual Hosting module (nsvirtual)
 * Copyright (C) 2001 TONIB Ltd.
 * maav <maav@bastun.net>
 *
 * AOLserver    http://www.aolserver.com/
 *
 * This was written and tested with:
 * AOLserver 3.3 pre2, MySQL 3.23.27-beta, Linux 2.4.0-pre2 glibc 2.1.
 * 
 * Version 1.1
 *
 */

#include "ns.h"
#include <string.h>

#define MODULE_NAME "nsvirtualhost"
#define SELECT1 "select hostname,path from vhost"

NS_EXPORT int Ns_ModuleVersion = 1;
static char *dbPoolName;
static Ns_Set *map;

static Ns_UrlToFileProc MappedUrlToFile;
static Ns_Callback TimedDbAppend;

int Ns_ModuleInit(char *hServer, char *hModule) {

  char *moduleConfigPath, *serverConfigPath;
  Ns_Set *row;
  int n, status;
  Ns_DbHandle *dbhandle;

  serverConfigPath = Ns_ConfigGetPath(hServer, NULL, NULL);
  moduleConfigPath = Ns_ConfigGetPath(hServer, hModule, NULL);

  if (Ns_ConfigGetInt(moduleConfigPath,"RefreshTime", &n) != NS_TRUE) {
        n = 300;
        Ns_Log(Warning, "Ns_ModuleInit(%s,%s): No RefreshTime directive found in config file", hServer, hModule, n);
  }
  if ((dbPoolName = Ns_ConfigGetValueExact(moduleConfigPath, "dbPoolName")) == NULL) {
        dbPoolName = "mysqldb";
        Ns_Log(Warning, "Ns_ModuleInit(%s,%s): No dbPoolName directive found in config file", hServer, hModule);
  }

  if (Ns_DbPoolAllowable(hServer, dbPoolName) == NS_FALSE) {
	Ns_Fatal("nsvirtualhost: Database pool (%s) is not available", dbPoolName);
  }
  
  map = Ns_SetCreate("");
  if ((dbhandle = Ns_DbPoolGetHandle(dbPoolName)) != NULL) {
	row = Ns_DbSelect(dbhandle, SELECT1); 
	if (row == NULL) {
		Ns_Fatal("nsvirtualhost: Cannot obtain database handle");
	}
	while ((status = Ns_DbGetRow(dbhandle, row)) == NS_OK) {
		Ns_SetPut(map, Ns_SetGet(row, Ns_SetKey(row, 0)), Ns_SetGet(row, Ns_SetKey(row, 1)));
	}
	if (status != NS_END_DATA) {
		Ns_Fatal("nsvirtualhost: Cannot get the SQL request");
	}

  Ns_DbPoolPutHandle(dbhandle); 
  }

  Ns_SetUrlToFileProc(hServer, MappedUrlToFile);
  Ns_ScheduleProc(TimedDbAppend, NULL, 0, n);

  return NS_OK;
}

static int MappedUrlToFile(Ns_DString *dest, char *hServer, char *url) {

  Ns_Conn *conn;
  char *hstname;

  conn = Ns_GetConn();
  if ((hstname = Ns_SetGet(conn->headers, "Host")) != NULL) {

  	if (strchr(hstname, '@') != NULL) {
		Ns_Strtok(hstname, "@");
        	hstname = Ns_Strtok(NULL, "@");
  	}

	  if (strchr(hstname, ':') != NULL) {
		hstname = Ns_Strtok(hstname, ":");
  	}

	  Ns_MakePath(dest, Ns_SetIGet(map, hstname), conn->request->url, NULL);
  } else Ns_MakePath(dest, Ns_PageRoot(hServer), url, NULL);

return NS_OK;
}

static void TimedDbAppend(void *ignored) {

  Ns_Set *row = NULL, *newmap = NULL, *oldmap = NULL;
  Ns_DbHandle *dbhandle;
  int status;

  if ((dbhandle = Ns_DbPoolGetHandle(dbPoolName)) != NULL) {
	row = Ns_DbSelect(dbhandle, SELECT1);
	if (row == NULL) {
		Ns_Log(Error, "nsvirtualhost: Error getting database handle during update");
		Ns_DbPoolPutHandle(dbhandle);
		return;
	}
		newmap = Ns_SetCreate("");
		while ((status = Ns_DbGetRow(dbhandle, row)) == NS_OK) {
			Ns_SetPut(newmap, Ns_SetGet(row, Ns_SetKey(row, 0)), Ns_SetGet(row, Ns_SetKey(row, 1)));
		}
		if (status != NS_END_DATA) {
			Ns_Log(Error, "nsvirtualhost: Error getting SQL request during update");
			Ns_SetFree(newmap);
			Ns_DbPoolPutHandle(dbhandle);
			return;
		}
		oldmap = map;
		map = Ns_SetCopy(newmap);
		Ns_SetFree(newmap); Ns_SetFree(oldmap);
  Ns_DbPoolPutHandle(dbhandle);
  }
}
